@@ -1,4 +1,4 @@
-/*	$OpenBSD: mail.local.c,v 1.39 2020/02/09 14:59:20 millert Exp $	*/
+/*	$OpenBSD: mail.local.c,v 1.41 2023/05/15 12:02:40 op Exp $	*/
 
 /*-
  * Copyright (c) 1996-1998 Theo de Raadt <deraadt@theos.com>
@@ -34,7 +34,6 @@
 #include "includes.h"
 
 #include <sys/types.h>
-
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <sys/wait.h>
@@ -249,7 +248,8 @@ retry:
 	}
 
 	curoff = lseek(mbfd, 0, SEEK_END);
-	(void)snprintf(biffmsg, sizeof biffmsg, "%s@%lld\n", name, curoff);
+	(void)snprintf(biffmsg, sizeof biffmsg, "%s@%lld\n", name,
+	    (long long)curoff);
 	if (lseek(fd, 0, SEEK_SET) == (off_t)-1) {
 		mwarn("temporary file: %s", strerror(errno));
 		goto bad;
