@@ -92,6 +92,10 @@ int	scan_scaled(char *, long long *);
 const char *inet_ntop(int af, const void *src, char *dst, socklen_t size);
 #endif
 
+#ifndef HAVE_RES_RANDOMID
+unsigned int res_randomid(void);
+#endif
+
 #ifndef HAVE_STRSEP
 char *strsep(char **stringp, const char *delim);
 #endif
@@ -210,10 +214,6 @@ void freezero(void *, size_t);
 int pidfile(const char *basename);
 #endif
 
-#ifndef HAVE_PW_DUP
-struct passwd *pw_dup(const struct passwd *);
-#endif
-
 #if !defined(HAVE_REALLOCARRAY) || defined(LIBRESSL_VERSION_NUMBER)
 void *reallocarray(void *, size_t, size_t);
 #endif
@@ -227,7 +227,7 @@ __attribute__ ((noreturn))
 void errc(int, int, const char *, ...);
 #endif
 
-#ifndef HAVE_INET_NET_PTON
+#if !defined(HAVE_INET_NET_PTON) || defined(HAVE_LIBBSD)
 int inet_net_pton(int, const char *, void *, size_t);
 #endif
 
@@ -263,6 +263,10 @@ int res_hnok(const char *);
 #define IPPORT_HILASTAUTO 65535
 #endif
 
+#if !HAVE_DECL_IPV6_ADDR_SCOPE_INTFACELOCAL
+#define USE_IPV6_ADDR_SCOPE_NODELOCAL 1
+#endif
+
 #ifndef HAVE_FLOCK
 int flock(int, int);
 #endif
@@ -288,7 +292,7 @@ char * strndup(const char *, size_t);
 #endif
 
 #ifndef HAVE_STRNLEN
-char * strnlen(const char *, size_t);
+size_t strnlen(const char *, size_t);
 #endif
 
 #ifndef HAVE_STRUCT_TIMEVAL
@@ -298,6 +302,10 @@ struct timeval {
 }
 #endif
 
+#ifndef HAVE_TIMINGSAFE_MEMCMP
+int timingsafe_memcmp(const void *, const void *, size_t);
+#endif
+
 #ifdef NEED_NANOSLEEP
 #ifndef HAVE_STRUCT_TIMESPEC
 struct timespec {
@@ -339,4 +347,20 @@ int pipe2(int pipefd[2], int flags);
 char *get_progname(char *);
 
 
+#ifdef NEED_CLOCK_GETTIME
+#ifndef CLOCK_REALTIME
+#define CLOCK_REALTIME	0
+#endif
+#ifndef	CLOCK_MONOTONIC
+#define CLOCK_MONOTONIC	3
+#endif
+int	clock_gettime(int clock_id, struct timespec *tp);
+#endif
+
+#ifdef NEED_GETSUBOPT
+#define getsubopt(optionp, tockens, valuep) openbsd_getsubopt(optionp, tockens, valuep)
+int openbsd_getsubopt(char **optionp, char * const *tokens, char **valuep);
+extern  char *suboptarg;               /* getsubopt(3) external variable */
+#endif
+
 #endif /* _OPENBSD_COMPAT_H */
