@@ -1,4 +1,4 @@
-/*	$OpenBSD: bounce.c,v 1.82 2020/04/24 11:34:07 eric Exp $	*/
+/*	$OpenBSD: bounce.c,v 1.89 2023/05/15 12:03:04 op Exp $	*/
 
 /*
  * Copyright (c) 2009 Gilles Chehade <gilles@poolp.org>
@@ -25,19 +25,12 @@
 #include <sys/tree.h>
 #include <sys/socket.h>
 
-#include <err.h>
 #include <errno.h>
-#include <event.h>
-#include <imsg.h>
 #include <inttypes.h>
-#include <pwd.h>
-#include <signal.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
-#include <limits.h>
 
 #include "smtpd.h"
 #include "log.h"
@@ -93,11 +86,13 @@ SPLAY_PROTOTYPE(bounce_message_tree, bounce_message, sp_entry,
     bounce_message_cmp);
 
 static void bounce_drain(void);
-static void bounce_send(struct bounce_session *, const char *, ...);
+static void bounce_send(struct bounce_session *, const char *, ...)
+	__attribute__((__format__ (printf, 2, 3)));
 static int  bounce_next_message(struct bounce_session *);
 static int  bounce_next(struct bounce_session *);
 static void bounce_delivery(struct bounce_message *, int, const char *);
-static void bounce_status(struct bounce_session *, const char *, ...);
+static void bounce_status(struct bounce_session *, const char *, ...)
+	__attribute__((__format__ (printf, 2, 3)));
 static void bounce_io(struct io *, int, void *);
 static void bounce_timeout(int, short, void *);
 static void bounce_free(struct bounce_session *);
@@ -143,7 +138,7 @@ bounce_add(uint64_t evpid)
 	}
 
 	if (evp.type != D_BOUNCE)
-		errx(1, "bounce: evp:%016" PRIx64 " is not of type D_BOUNCE!",
+		fatalx("bounce: evp:%016" PRIx64 " is not of type D_BOUNCE!",
 		    evp.id);
 
 	key.msgid = evpid_to_msgid(evpid);
@@ -249,7 +244,7 @@ bounce_timeout(int fd, short ev, void *arg)
 }
 
 static void
-bounce_drain()
+bounce_drain(void)
 {
 	struct bounce_message	*msg;
 	struct timeval		 tv;
@@ -292,7 +287,7 @@ bounce_drain()
 		}
 
 		log_debug("debug: bounce: requesting new enqueue socket...");
-		m_compose(p_pony, IMSG_QUEUE_SMTP_SESSION, 0, 0, -1, NULL, 0);
+		m_compose(p_dispatcher, IMSG_QUEUE_SMTP_SESSION, 0, 0, -1, NULL, 0);
 
 		running += 1;
 	}
@@ -318,7 +313,7 @@ bounce_send(struct bounce_session *s, const char *fmt, ...)
 }
 
 static const char *
-bounce_duration(long long int d)
+bounce_duration(long long d)
 {
 	static char buf[32];
 
