@@ -1,4 +1,4 @@
-/*	$OpenBSD: ca.c,v 1.36 2019/09/21 07:46:53 semarie Exp $	*/
+/*	$OpenBSD: ca.c,v 1.43 2023/03/26 18:11:48 tb Exp $	*/
 
 /*
  * Copyright (c) 2014 Reyk Floeter <reyk@openbsd.org>
@@ -28,20 +28,16 @@
 #include <err.h>
 #include <imsg.h>
 #include <limits.h>
+
+#include <openssl/err.h>
+#include <openssl/pem.h>
+#include <openssl/engine.h>
+
 #include <pwd.h>
 #include <signal.h>
-#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
-#include <openssl/ssl.h>
-#include <openssl/pem.h>
-#include <openssl/evp.h>
-#include <openssl/ecdsa.h>
-#include <openssl/rsa.h>
-#include <openssl/engine.h>
-#include <openssl/err.h>
-
 #include "smtpd.h"
 #include "log.h"
 #include "ssl.h"
@@ -71,6 +67,8 @@ static int ecdsae_sign_setup(EC_KEY *, BN_CTX *, BIGNUM **, BIGNUM **);
 static int ecdsae_do_verify(const unsigned char *, int, const ECDSA_SIG *,
     EC_KEY *);
 
+
+static struct dict pkeys;
 static uint64_t	 reqid = 0;
 
 static void
@@ -112,14 +110,14 @@ ca(void)
 
 	config_peer(PROC_CONTROL);
 	config_peer(PROC_PARENT);
-	config_peer(PROC_PONY);
+	config_peer(PROC_DISPATCHER);
 
 	/* Ignore them until we get our config */
-	mproc_disable(p_pony);
+	mproc_disable(p_dispatcher);
 
 #if HAVE_PLEDGE
 	if (pledge("stdio", NULL) == -1)
-		err(1, "pledge");
+		fatal("pledge");
 #endif
 
 	event_dispatch();
@@ -136,26 +134,29 @@ ca_init(void)
 	struct pki	*pki;
 	const char	*k;
 	void		*iter_dict;
+	char		*hash;
 
 	log_debug("debug: init private ssl-tree");
+	dict_init(&pkeys);
 	iter_dict = NULL;
 	while (dict_iter(env->sc_pki_dict, &iter_dict, &k, (void **)&pki)) {
 		if (pki->pki_key == NULL)
 			continue;
 
-		if ((in = BIO_new_mem_buf(pki->pki_key,
-		    pki->pki_key_len)) == NULL)
-			fatalx("ca_launch: key");
-
-		if ((pkey = PEM_read_bio_PrivateKey(in,
-		    NULL, NULL, NULL)) == NULL)
-			fatalx("ca_launch: PEM");
+		in = BIO_new_mem_buf(pki->pki_key, pki->pki_key_len);
+		if (in == NULL)
+			fatalx("ca_init: key");
+		pkey = PEM_read_bio_PrivateKey(in, NULL, NULL, NULL);
+		if (pkey == NULL)
+			fatalx("ca_init: PEM");
 		BIO_free(in);
 
-		pki->pki_pkey = pkey;
-
-		freezero(pki->pki_key, pki->pki_key_len);
-		pki->pki_key = NULL;
+		hash = ssl_pubkey_hash(pki->pki_cert, pki->pki_cert_len);
+		if (dict_check(&pkeys, hash))
+			EVP_PKEY_free(pkey);
+		else
+			dict_xset(&pkeys, hash, pkey);
+		free(hash);
 	}
 }
 
@@ -227,15 +228,15 @@ end:
 void
 ca_imsg(struct mproc *p, struct imsg *imsg)
 {
+	EVP_PKEY		*pkey;
 	RSA			*rsa = NULL;
 	EC_KEY			*ecdsa = NULL;
 	const void		*from = NULL;
 	unsigned char		*to = NULL;
 	struct msg		 m;
-	const char		*pkiname;
+	const char		*hash;
 	size_t			 flen, tlen, padding;
 	int			 buf_len;
-	struct pki		*pki;
 	int			 ret = 0;
 	uint64_t		 id;
 	int			 v;
@@ -250,7 +251,7 @@ ca_imsg(struct mproc *p, struct imsg *imsg)
 		ca_init();
 
 		/* Start fulfilling requests */
-		mproc_enable(p_pony);
+		mproc_enable(p_dispatcher);
 		return;
 
 	case IMSG_CTL_VERBOSE:
@@ -271,16 +272,15 @@ ca_imsg(struct mproc *p, struct imsg *imsg)
 	case IMSG_CA_RSA_PRIVDEC:
 		m_msg(&m, imsg);
 		m_get_id(&m, &id);
-		m_get_string(&m, &pkiname);
+		m_get_string(&m, &hash);
 		m_get_data(&m, &from, &flen);
 		m_get_size(&m, &tlen);
 		m_get_size(&m, &padding);
 		m_end(&m);
 
-		pki = dict_get(env->sc_pki_dict, pkiname);
-		if (pki == NULL || pki->pki_pkey == NULL ||
-		    (rsa = EVP_PKEY_get1_RSA(pki->pki_pkey)) == NULL)
-			fatalx("ca_imsg: invalid pki");
+		pkey = dict_get(&pkeys, hash);
+		if (pkey == NULL || (rsa = EVP_PKEY_get1_RSA(pkey)) == NULL)
+			fatalx("ca_imsg: invalid pkey hash");
 
 		if ((to = calloc(1, tlen)) == NULL)
 			fatalx("ca_imsg: calloc");
@@ -310,14 +310,14 @@ ca_imsg(struct mproc *p, struct imsg *imsg)
 	case IMSG_CA_ECDSA_SIGN:
 		m_msg(&m, imsg);
 		m_get_id(&m, &id);
-		m_get_string(&m, &pkiname);
+		m_get_string(&m, &hash);
 		m_get_data(&m, &from, &flen);
 		m_end(&m);
 
-		pki = dict_get(env->sc_pki_dict, pkiname);
-		if (pki == NULL || pki->pki_pkey == NULL ||
-		    (ecdsa = EVP_PKEY_get1_EC_KEY(pki->pki_pkey)) == NULL)
-			fatalx("ca_imsg: invalid pki");
+		pkey = dict_get(&pkeys, hash);
+		if (pkey == NULL ||
+		    (ecdsa = EVP_PKEY_get1_EC_KEY(pkey)) == NULL)
+			fatalx("ca_imsg: invalid pkey hash");
 
 		buf_len = ECDSA_size(ecdsa);
 		if ((to = calloc(1, buf_len)) == NULL)
@@ -334,7 +334,8 @@ ca_imsg(struct mproc *p, struct imsg *imsg)
 		EC_KEY_free(ecdsa);
 		return;
 	}
-	errx(1, "ca_imsg: unexpected %s imsg", imsg_to_str(imsg->hdr.type));
+
+	fatalx("ca_imsg: unexpected %s imsg", imsg_to_str(imsg->hdr.type));
 }
 
 /*
@@ -354,12 +355,12 @@ rsae_send_imsg(int flen, const unsigned char *from, unsigned char *to,
 	struct imsg	 imsg;
 	int		 n, done = 0;
 	const void	*toptr;
-	char		*pkiname;
+	char		*hash;
 	size_t		 tlen;
 	struct msg	 m;
 	uint64_t	 id;
 
-	if ((pkiname = RSA_get_ex_data(rsa, 0)) == NULL)
+	if ((hash = RSA_get_ex_data(rsa, 0)) == NULL)
 		return (0);
 
 	/*
@@ -369,7 +370,7 @@ rsae_send_imsg(int flen, const unsigned char *from, unsigned char *to,
 	m_create(p_ca, cmd, 0, 0, -1);
 	reqid++;
 	m_add_id(p_ca, reqid);
-	m_add_string(p_ca, pkiname);
+	m_add_string(p_ca, hash);
 	m_add_data(p_ca, (const void *)from, (size_t)flen);
 	m_add_size(p_ca, (size_t)RSA_size(rsa));
 	m_add_size(p_ca, (size_t)padding);
@@ -389,7 +390,7 @@ rsae_send_imsg(int flen, const unsigned char *from, unsigned char *to,
 			if (n == 0)
 				break;
 
-			log_imsg(PROC_PONY, PROC_CA, &imsg);
+			log_imsg(PROC_DISPATCHER, PROC_CA, &imsg);
 
 			switch (imsg.hdr.type) {
 			case IMSG_CA_RSA_PRIVENC:
@@ -397,7 +398,7 @@ rsae_send_imsg(int flen, const unsigned char *from, unsigned char *to,
 				break;
 			default:
 				/* Another imsg is queued up in the buffer */
-				pony_imsg(p_ca, &imsg);
+				dispatcher_imsg(p_ca, &imsg);
 				imsg_free(&imsg);
 				continue;
 			}
@@ -546,17 +547,17 @@ ecdsae_send_enc_imsg(const unsigned char *dgst, int dgst_len,
 	struct imsg	 imsg;
 	int		 n, done = 0;
 	const void	*toptr;
-	char		*pkiname;
+	char		*hash;
 	size_t		 tlen;
 	struct msg	 m;
 	uint64_t	 id;
 	ECDSA_SIG	*sig = NULL;
 
 #if defined(SUPPORT_ECDSA)
-	if ((pkiname = ECDSA_get_ex_data(eckey, 0)) == NULL)
+	if ((hash = ECDSA_get_ex_data(eckey, 0)) == NULL)
 		return (0);
 #else
-	if ((pkiname = EC_KEY_get_ex_data(eckey, 0)) == NULL)
+	if ((hash = EC_KEY_get_ex_data(eckey, 0)) == NULL)
 		return (0);
 #endif
 
@@ -567,7 +568,7 @@ ecdsae_send_enc_imsg(const unsigned char *dgst, int dgst_len,
 	m_create(p_ca, IMSG_CA_ECDSA_SIGN, 0, 0, -1);
 	reqid++;
 	m_add_id(p_ca, reqid);
-	m_add_string(p_ca, pkiname);
+	m_add_string(p_ca, hash);
 	m_add_data(p_ca, (const void *)dgst, (size_t)dgst_len);
 	m_flush(p_ca);
 
@@ -584,14 +585,14 @@ ecdsae_send_enc_imsg(const unsigned char *dgst, int dgst_len,
 			if (n == 0)
 				break;
 
-			log_imsg(PROC_PONY, PROC_CA, &imsg);
+			log_imsg(PROC_DISPATCHER, PROC_CA, &imsg);
 
 			switch (imsg.hdr.type) {
 			case IMSG_CA_ECDSA_SIGN:
 				break;
 			default:
 				/* Another imsg is queued up in the buffer */
-				pony_imsg(p_ca, &imsg);
+				dispatcher_imsg(p_ca, &imsg);
 				imsg_free(&imsg);
 				continue;
 			}
