@@ -1,4 +1,4 @@
-/*	$OpenBSD: control.c,v 1.125 2020/09/23 19:11:50 martijn Exp $	*/
+/*	$OpenBSD: control.c,v 1.129 2023/03/08 04:43:15 guenther Exp $	*/
 
 /*
  * Copyright (c) 2012 Gilles Chehade <gilles@poolp.org>
@@ -24,10 +24,8 @@
 #include <sys/queue.h>
 #include <sys/tree.h>
 #include <sys/stat.h>
-#include <sys/socket.h>
 #include <sys/un.h>
 
-#include <err.h>
 #include <errno.h>
 #include <event.h>
 #include <fcntl.h>
@@ -35,12 +33,10 @@
 #include <imsg.h>
 #include <pwd.h>
 #include <signal.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
-#include <limits.h>
 
 #include "smtpd.h"
 #include "log.h"
@@ -162,7 +158,7 @@ control_imsg(struct mproc *p, struct imsg *imsg)
 		return;
 	}
 
-	errx(1, "control_imsg: unexpected %s imsg",
+	fatalx("control_imsg: unexpected %s imsg",
 	    imsg_to_str(imsg->hdr.type));
 }
 
@@ -251,14 +247,14 @@ control(void)
 	config_peer(PROC_QUEUE);
 	config_peer(PROC_PARENT);
 	config_peer(PROC_LKA);
-	config_peer(PROC_PONY);
+	config_peer(PROC_DISPATCHER);
 	config_peer(PROC_CA);
 
 	control_listen();
 
 #if HAVE_PLEDGE
 	if (pledge("stdio unix recvfd sendfd", NULL) == -1)
-		err(1, "pledge");
+		fatal("pledge");
 #endif
 
 	event_dispatch();
@@ -285,7 +281,6 @@ control_listen(void)
 	event_add(&control_state.ev, NULL);
 }
 
-/* ARGSUSED */
 static void
 control_accept(int listenfd, short event, void *arg)
 {
@@ -433,7 +428,6 @@ control_digest_update(const char *key, size_t value, int incr)
 	}
 }
 
-/* ARGSUSED */
 static void
 control_dispatch_ext(struct mproc *p, struct imsg *imsg)
 {
@@ -465,7 +459,7 @@ control_dispatch_ext(struct mproc *p, struct imsg *imsg)
 			m_compose(p, IMSG_CTL_FAIL, 0, 0, -1, NULL, 0);
 			return;
 		}
-		m_compose(p_pony, IMSG_CTL_SMTP_SESSION, c->id, 0, -1,
+		m_compose(p_dispatcher, IMSG_CTL_SMTP_SESSION, c->id, 0, -1,
 		    &c->euid, sizeof(c->euid));
 		return;
 
@@ -612,7 +606,7 @@ control_dispatch_ext(struct mproc *p, struct imsg *imsg)
 		}
 		log_info("info: smtp paused");
 		env->sc_flags |= SMTPD_SMTP_PAUSED;
-		m_compose(p_pony, IMSG_CTL_PAUSE_SMTP, 0, 0, -1, NULL, 0);
+		m_compose(p_dispatcher, IMSG_CTL_PAUSE_SMTP, 0, 0, -1, NULL, 0);
 		m_compose(p, IMSG_CTL_OK, 0, 0, -1, NULL, 0);
 		return;
 
@@ -662,7 +656,7 @@ control_dispatch_ext(struct mproc *p, struct imsg *imsg)
 		}
 		log_info("info: smtp resumed");
 		env->sc_flags &= ~SMTPD_SMTP_PAUSED;
-		m_forward(p_pony, imsg);
+		m_forward(p_dispatcher, imsg);
 		m_compose(p, IMSG_CTL_OK, 0, 0, -1, NULL, 0);
 		return;
 
@@ -670,7 +664,7 @@ control_dispatch_ext(struct mproc *p, struct imsg *imsg)
 		if (c->euid)
 			goto badcred;
 
-		m_forward(p_pony, imsg);
+		m_forward(p_dispatcher, imsg);
 		m_compose(p, IMSG_CTL_OK, 0, 0, -1, NULL, 0);
 		return;
 
@@ -697,7 +691,7 @@ control_dispatch_ext(struct mproc *p, struct imsg *imsg)
 			goto badcred;
 
 		imsg->hdr.peerid = c->id;
-		m_forward(p_pony, imsg);
+		m_forward(p_dispatcher, imsg);
 		return;
 
 	case IMSG_CTL_SHOW_STATUS:
@@ -716,10 +710,10 @@ control_dispatch_ext(struct mproc *p, struct imsg *imsg)
 		if (imsg->hdr.len - IMSG_HEADER_SIZE <= sizeof(ss))
 			goto invalid;
 		memmove(&ss, imsg->data, sizeof(ss));
-		m_create(p_pony, imsg->hdr.type, c->id, 0, -1);
-		m_add_sockaddr(p_pony, (struct sockaddr *)&ss);
-		m_add_string(p_pony, (char *)imsg->data + sizeof(ss));
-		m_close(p_pony);
+		m_create(p_dispatcher, imsg->hdr.type, c->id, 0, -1);
+		m_add_sockaddr(p_dispatcher, (struct sockaddr *)&ss);
+		m_add_string(p_dispatcher, (char *)imsg->data + sizeof(ss));
+		m_close(p_dispatcher);
 		return;
 
 	case IMSG_CTL_SCHEDULE:
@@ -795,9 +789,9 @@ control_broadcast_verbose(int msg, int v)
 	m_add_int(p_lka, v);
 	m_close(p_lka);
 
-	m_create(p_pony, msg, 0, 0, -1);
-	m_add_int(p_pony, v);
-	m_close(p_pony);
+	m_create(p_dispatcher, msg, 0, 0, -1);
+	m_add_int(p_dispatcher, v);
+	m_close(p_dispatcher);
 
 	m_create(p_queue, msg, 0, 0, -1);
 	m_add_int(p_queue, v);
