@@ -1,4 +1,4 @@
-/*	$OpenBSD: enqueue.c,v 1.118 2020/03/18 20:17:14 eric Exp $	*/
+/*	$OpenBSD: enqueue.c,v 1.120 2021/07/28 19:39:50 benno Exp $	*/
 
 /*
  * Copyright (c) 2005 Henning Brauer <henning@bulabula.org>
@@ -29,18 +29,11 @@
 #include <ctype.h>
 #include <err.h>
 #include <errno.h>
-#include <event.h>
-#include <grp.h>
-#include <imsg.h>
-#include <inttypes.h>
 #include <pwd.h>
-#include <stdarg.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
-#include <limits.h>
 
 #include "smtpd.h"
 
@@ -55,7 +48,8 @@ static char *qualify_addr(char *);
 static void rcpt_add(char *);
 static int open_connection(void);
 static int get_responses(FILE *, int);
-static int send_line(FILE *, int, char *, ...);
+static int send_line(FILE *, int, char *, ...)
+    __attribute__((__format__ (printf, 3, 4)));
 static int enqueue_offline(int, char *[], FILE *, FILE *);
 static int savedeadletter(struct passwd *, FILE *);
 
@@ -561,12 +555,12 @@ build_from(char *fake_from, struct passwd *pw)
 			    apos, pw->pw_gecos,
 			    pw->pw_name,
 			    len - apos - 1, p + 1) == -1)
-				err(1, "asprintf");
+				err(1, NULL);
 			msg.fromname[apos] = toupper((unsigned char)msg.fromname[apos]);
 		} else {
 			if (asprintf(&msg.fromname, "%.*s", len,
 			    pw->pw_gecos) == -1)
-				err(1, "asprintf");
+				err(1, NULL);
 		}
 	}
 }
