@@ -1,4 +1,4 @@
-/*	$OpenBSD: mta.c,v 1.234 2019/12/21 10:34:07 gilles Exp $	*/
+/*	$OpenBSD: mta.c,v 1.244 2023/05/16 17:48:52 op Exp $	*/
 
 /*
  * Copyright (c) 2008 Pierre-Yves Ritschard <pyr@openbsd.org>
@@ -27,7 +27,6 @@
 #include <sys/socket.h>
 
 #include <ctype.h>
-#include <err.h>
 #include <errno.h>
 #include <event.h>
 #include <imsg.h>
@@ -41,10 +40,11 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
-#include <unistd.h>
+#include <tls.h>
 
 #include "smtpd.h"
 #include "log.h"
+#include "ssl.h"
 
 #define MAXERROR_PER_ROUTE	4
 
@@ -60,6 +60,7 @@
 #define RELAY_ONHOLD		0x01
 #define RELAY_HOLDQ		0x02
 
+static void mta_setup_dispatcher(struct dispatcher *);
 static void mta_handle_envelope(struct envelope *, const char *);
 static void mta_query_smarthost(struct envelope *);
 static void mta_on_smarthost(struct envelope *, const char *);
@@ -141,6 +142,12 @@ int mta_is_blocked(struct mta_source *, char *);
 static int mta_block_cmp(const struct mta_block *, const struct mta_block *);
 SPLAY_PROTOTYPE(mta_block_tree, mta_block, entry, mta_block_cmp);
 
+/*
+ * This function is not publicy exported because it is a hack until libtls
+ * has a proper privsep setup
+ */
+void tls_config_use_fake_private_key(struct tls_config *config);
+
 static struct mta_relay_tree		relays;
 static struct mta_domain_tree		domains;
 static struct mta_host_tree		hosts;
@@ -460,12 +467,91 @@ mta_imsg(struct mproc *p, struct imsg *imsg)
 		return;
 	}
 
-	errx(1, "mta_imsg: unexpected %s imsg", imsg_to_str(imsg->hdr.type));
+	fatalx("mta_imsg: unexpected %s imsg", imsg_to_str(imsg->hdr.type));
 }
 
 void
 mta_postfork(void)
 {
+	struct dispatcher *dispatcher;
+	const char *key;
+	void *iter;
+
+	iter = NULL;
+	while (dict_iter(env->sc_dispatchers, &iter, &key, (void **)&dispatcher)) {
+		log_debug("%s: %s", __func__, key);
+		mta_setup_dispatcher(dispatcher);
+	}
+}
+
+static void
+mta_setup_dispatcher(struct dispatcher *dispatcher)
+{
+	struct dispatcher_remote *remote;
+	static const char *dheparams[] = { "none", "auto", "legacy" };
+	struct tls_config *config;
+	struct pki *pki;
+	struct ca *ca;
+	const char *ciphers;
+	uint32_t protos;
+
+	if (dispatcher->type != DISPATCHER_REMOTE)
+		return;
+
+	remote = &dispatcher->u.remote;
+
+	if ((config = tls_config_new()) == NULL)
+		fatal("smtpd: tls_config_new");
+
+	ciphers = env->sc_tls_ciphers;
+	if (remote->tls_ciphers)
+		ciphers = remote->tls_ciphers;
+	if (ciphers && tls_config_set_ciphers(config, ciphers) == -1)
+		fatalx("%s", tls_config_error(config));
+
+	if (remote->tls_protocols) {
+		if (tls_config_parse_protocols(&protos,
+		    remote->tls_protocols) == -1)
+			fatalx("failed to parse protocols \"%s\"",
+			    remote->tls_protocols);
+		if (tls_config_set_protocols(config, protos) == -1)
+			fatalx("%s", tls_config_error(config));
+	}
+
+	if (remote->pki) {
+		pki = dict_get(env->sc_pki_dict, remote->pki);
+		if (pki == NULL)
+			fatalx("client pki \"%s\" not found", remote->pki);
+
+		tls_config_set_dheparams(config, dheparams[pki->pki_dhe]);
+		tls_config_use_fake_private_key(config);
+		if (tls_config_set_keypair_mem(config, pki->pki_cert,
+		    pki->pki_cert_len, NULL, 0) == -1)
+			fatalx("tls_config_set_keypair_mem: %s",
+			    tls_config_error(config));
+	}
+
+	if (remote->ca) {
+		ca = dict_get(env->sc_ca_dict, remote->ca);
+		if (tls_config_set_ca_mem(config, ca->ca_cert, ca->ca_cert_len)
+		    == -1)
+			fatalx("tls_config_set_ca_mem: %s",
+			    tls_config_error(config));
+	}
+	else if (tls_config_set_ca_file(config, tls_default_ca_cert_file())
+	    == -1)
+		fatalx("tls_config_set_ca_file: %s",
+		    tls_config_error(config));
+
+	if (remote->tls_verify) {
+		tls_config_verify(config);
+	} else {
+		tls_config_insecure_noverifycert(config);
+		tls_config_insecure_noverifyname(config);
+		tls_config_insecure_noverifytime(config);
+	}
+
+	remote->tls_config = config;
 }
 
 void
@@ -1467,7 +1553,7 @@ mta_flush(struct mta_relay *relay, int fail, const char *error)
 	    mta_relay_to_text(relay), fail, error);
 
 	if (fail != IMSG_MTA_DELIVERY_TEMPFAIL && fail != IMSG_MTA_DELIVERY_PERMFAIL)
-		errx(1, "unexpected delivery status %d", fail);
+		fatalx("unexpected delivery status %d", fail);
 
 	n = 0;
 	while ((task = TAILQ_FIRST(&relay->tasks))) {
@@ -1763,10 +1849,6 @@ mta_relay(struct envelope *e, struct relayhost *relayh)
 	if (!key.authlabel[0])
 		key.authlabel = NULL;
 
-	if ((key.tls == RELAY_TLS_STARTTLS || key.tls == RELAY_TLS_SMTPS) &&
-	    dispatcher->u.remote.tls_noverify == 0)
-		key.flags |= RELAY_TLS_VERIFY;
-
 	if ((r = SPLAY_FIND(mta_relay_tree, &relays, &key)) == NULL) {
 		r = xcalloc(1, sizeof *r);
 		TAILQ_INIT(&r->tasks);
