@@ -1,4 +1,4 @@
-/*	$OpenBSD: mta_session.c,v 1.138 2020/12/21 11:44:07 martijn Exp $	*/
+/*	$OpenBSD: mta_session.c,v 1.148 2023/05/15 12:03:04 op Exp $	*/
 
 /*
  * Copyright (c) 2008 Pierre-Yves Ritschard <pyr@openbsd.org>
@@ -26,29 +26,18 @@
 #include <sys/tree.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
-#include <sys/uio.h>
 
 #include <ctype.h>
-#include <err.h>
 #include <errno.h>
-#include <event.h>
-#include <imsg.h>
 #include <inttypes.h>
-#include <netdb.h>
-#include <openssl/ssl.h>
-#include <pwd.h>
-#include <resolv.h>
-#include <limits.h>
-#include <signal.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <tls.h>
 #include <unistd.h>
 
 #include "smtpd.h"
 #include "log.h"
-#include "ssl.h"
 
 #define MAX_TRYBEFOREDISABLE	10
 
@@ -149,16 +138,15 @@ static void mta_on_timeout(struct runq *, void *);
 static void mta_connect(struct mta_session *);
 static void mta_enter_state(struct mta_session *, int);
 static void mta_flush_task(struct mta_session *, int, const char *, size_t, int);
-static void mta_error(struct mta_session *, const char *, ...);
-static void mta_send(struct mta_session *, char *, ...);
+static void mta_error(struct mta_session *, const char *, ...)
+    __attribute__((__format__ (printf, 2, 3)));
+static void mta_send(struct mta_session *, char *, ...)
+    __attribute__((__format__ (printf, 2, 3)));
 static ssize_t mta_queue_data(struct mta_session *);
 static void mta_response(struct mta_session *, char *);
 static const char * mta_strstate(int);
-static void mta_cert_init(struct mta_session *);
-static void mta_cert_init_cb(void *, int, const char *, const void *, size_t);
-static void mta_cert_verify(struct mta_session *);
-static void mta_cert_verify_cb(void *, int);
-static void mta_tls_verified(struct mta_session *);
+static void mta_tls_init(struct mta_session *);
+static void mta_tls_started(struct mta_session *);
 static struct mta_session *mta_tree_pop(struct tree *, uint64_t);
 static const char * dsn_strret(enum dsn_ret);
 static const char * dsn_strnotify(uint8_t);
@@ -361,7 +349,7 @@ mta_session_imsg(struct mproc *p, struct imsg *imsg)
 		return;
 
 	default:
-		errx(1, "mta_session_imsg: unexpected %s imsg",
+		fatalx("mta_session_imsg: unexpected %s imsg",
 		    imsg_to_str(imsg->hdr.type));
 	}
 }
@@ -975,7 +963,7 @@ mta_response(struct mta_session *s, char *line)
 			return;
 		}
 
-		mta_cert_init(s);
+		mta_tls_init(s);
 		break;
 
 	case MTA_AUTH_PLAIN:
@@ -1176,7 +1164,7 @@ mta_response(struct mta_session *s, char *line)
 			s->rcptcount = 0;
 			if (s->relay->limits->sessdelay_transaction) {
 				log_debug("debug: mta: waiting for %llds before next transaction",
-				    (long long int)s->relay->limits->sessdelay_transaction);
+				    (long long)s->relay->limits->sessdelay_transaction);
 				s->hangon = s->relay->limits->sessdelay_transaction -1;
 				s->flags |= MTA_HANGON;
 				runq_schedule(hangon,
@@ -1196,7 +1184,7 @@ mta_response(struct mta_session *s, char *line)
 		}
 		if (s->relay->limits->sessdelay_transaction) {
 			log_debug("debug: mta: waiting for %llds after reset",
-			    (long long int)s->relay->limits->sessdelay_transaction);
+			    (long long)s->relay->limits->sessdelay_transaction);
 			s->hangon = s->relay->limits->sessdelay_transaction -1;
 			s->flags |= MTA_HANGON;
 			runq_schedule(hangon,
@@ -1230,7 +1218,9 @@ mta_io(struct io *io, int evt, void *arg)
 
 		if (s->use_smtps) {
 			io_set_write(io);
-			mta_cert_init(s);
+			mta_tls_init(s);
+			if (s->flags & MTA_FREE)
+				mta_free(s);
 		}
 		else {
 			mta_enter_state(s, MTA_BANNER);
@@ -1240,13 +1230,14 @@ mta_io(struct io *io, int evt, void *arg)
 
 	case IO_TLSREADY:
 		log_info("%016"PRIx64" mta tls ciphers=%s",
-		    s->id, ssl_to_text(io_tls(s->io)));
+		    s->id, tls_to_text(io_tls(s->io)));
 		s->flags |= MTA_TLS;
+		if (s->relay->dispatcher->u.remote.tls_verify)
+			s->flags |= MTA_TLS_VERIFIED;
 
+		mta_tls_started(s);
 		mta_report_link_tls(s,
-		    ssl_to_text(io_tls(s->io)));
-
-		mta_cert_verify(s);
+		    tls_to_text(io_tls(s->io)));
 		break;
 
 	case IO_DATAIN:
@@ -1379,7 +1370,6 @@ mta_io(struct io *io, int evt, void *arg)
 		break;
 
 	case IO_ERROR:
-	case IO_TLSERROR:
 		log_debug("debug: mta: %p: IO error: %s", s, io_error(io));
 
 		if (s->state == MTA_STARTTLS && s->use_smtp_tls) {
@@ -1580,132 +1570,52 @@ mta_error(struct mta_session *s, const char *fmt, ...)
 }
 
 static void
-mta_cert_init(struct mta_session *s)
+mta_tls_init(struct mta_session *s)
 {
-	const char *name;
-	int fallback;
+	struct dispatcher_remote *remote;
+	struct tls *tls;
 
-	if (s->relay->pki_name) {
-		name = s->relay->pki_name;
-		fallback = 0;
-	}
-	else {
-		name = s->helo;
-		fallback = 1;
-	}
-
-	if (cert_init(name, fallback, mta_cert_init_cb, s)) {
-		tree_xset(&wait_tls_init, s->id, s);
-		s->flags |= MTA_WAIT;
-	}
-}
-
-static void
-mta_cert_init_cb(void *arg, int status, const char *name, const void *cert,
-    size_t cert_len)
-{
-	struct mta_session *s = arg;
-	void *ssl;
-	char *xname = NULL, *xcert = NULL;
-
-	if (s->flags & MTA_WAIT)
-		mta_tree_pop(&wait_tls_init, s->id);
-
-	if (status == CA_FAIL && s->relay->pki_name) {
-		log_info("%016"PRIx64" mta closing reason=ca-failure", s->id);
-		mta_free(s);
+	if ((tls = tls_client()) == NULL) {
+		log_info("%016"PRIx64" mta closing reason=tls-failure", s->id);
+		s->flags |= MTA_FREE;
 		return;
 	}
 
-	if (name)
-		xname = xstrdup(name);
-	if (cert)
-		xcert = xmemdup(cert, cert_len);
-	ssl = ssl_mta_init(xname, xcert, cert_len, env->sc_tls_ciphers);
-	free(xname);
-	free(xcert);
-	if (ssl == NULL)
-		fatal("mta: ssl_mta_init");
-	io_start_tls(s->io, ssl);
-}
-
-static void
-mta_cert_verify(struct mta_session *s)
-{
-	const char *name;
-	int fallback;
-
-	if (s->relay->ca_name) {
-		name = s->relay->ca_name;
-		fallback = 0;
+	remote = &s->relay->dispatcher->u.remote;
+	if ((s->flags & MTA_WANT_SECURE) && !remote->tls_required) {
+		/* If TLS not explicitly configured, use implicit config. */
+		remote->tls_required = 1;
+		remote->tls_verify = 1;
+		tls_config_verify(remote->tls_config);
 	}
-	else {
-		name = s->helo;
-		fallback = 1;
-	}
-
-	if (cert_verify(io_tls(s->io), name, fallback, mta_cert_verify_cb, s)) {
-		tree_xset(&wait_tls_verify, s->id, s);
-		io_pause(s->io, IO_IN);
-		s->flags |= MTA_WAIT;
-	}
-}
-
-static void
-mta_cert_verify_cb(void *arg, int status)
-{
-	struct mta_session *s = arg;
-	int match, resume = 0;
-	X509 *cert;
-
-	if (s->flags & MTA_WAIT) {
-		mta_tree_pop(&wait_tls_verify, s->id);
-		resume = 1;
-	}
-
-	if (status == CERT_OK) {
-		cert = SSL_get_peer_certificate(io_tls(s->io));
-		if (!cert)
-			status = CERT_NOCERT;
-		else {
-			match = 0;
-			(void)ssl_check_name(cert, s->mxname, &match);
-			X509_free(cert);
-			if (!match) {
-				log_info("%016"PRIx64" mta "
-				    "ssl_check_name: no match for '%s' in cert",
-				    s->id, s->mxname);
-				status = CERT_INVALID;
-			}
-		}
-	}
-
-	if (status == CERT_OK)
-		s->flags |= MTA_TLS_VERIFIED;
-	else if (s->relay->flags & RELAY_TLS_VERIFY) {
-		errno = 0;
-		mta_error(s, "SSL certificate check failed");
-		mta_free(s);
+	if (tls_configure(tls, remote->tls_config) == -1) {
+		log_info("%016"PRIx64" mta closing reason=tls-failure", s->id);
+		tls_free(tls);
+		s->flags |= MTA_FREE;
 		return;
 	}
 
-	mta_tls_verified(s);
-	if (resume)
-		io_resume(s->io, IO_IN);
+	if (io_connect_tls(s->io, tls, s->mxname) == -1) {
+		log_info("%016"PRIx64" mta closing reason=tls-connect-failed", s->id);
+		tls_free(tls);
+		s->flags |= MTA_FREE;
+	}
 }
 
 static void
-mta_tls_verified(struct mta_session *s)
+mta_tls_started(struct mta_session *s)
 {
-	X509 *x;
-
-	x = SSL_get_peer_certificate(io_tls(s->io));
-	if (x) {
-	  log_info("%016"PRIx64" mta "
-		   "server-cert-check result=\"%s\"",
-		   s->id,
-		   (s->flags & MTA_TLS_VERIFIED) ? "success" : "failure");
-		X509_free(x);
+	if (tls_peer_cert_provided(io_tls(s->io))) {
+		log_info("%016"PRIx64" mta "
+		    "cert-check result=\"%s\" fingerprint=\"%s\"",
+		    s->id,
+		    (s->flags & MTA_TLS_VERIFIED) ? "valid" : "unverified",
+		    tls_peer_cert_hash(io_tls(s->io)));
+	}
+	else {
+		log_info("%016"PRIx64" smtp "
+		    "cert-check result=\"no certificate presented\"",
+		    s->id);
 	}
 
 	if (s->use_smtps) {
