@@ -1,4 +1,4 @@
-/*	$OpenBSD: queue.c,v 1.190 2020/04/22 11:35:34 eric Exp $	*/
+/*	$OpenBSD: queue.c,v 1.194 2021/08/02 12:33:34 eric Exp $	*/
 
 /*
  * Copyright (c) 2008 Gilles Chehade <gilles@poolp.org>
@@ -26,7 +26,6 @@
 #include <sys/socket.h>
 #include <sys/stat.h>
 
-#include <err.h>
 #include <event.h>
 #include <fcntl.h>
 #include <grp.h> /* needed for setgroups */
@@ -34,12 +33,10 @@
 #include <inttypes.h>
 #include <pwd.h>
 #include <signal.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
-#include <limits.h>
 
 #include "smtpd.h"
 #include "log.h"
@@ -156,15 +153,15 @@ queue_imsg(struct mproc *p, struct imsg *imsg)
 			log_warnx("warn: imsg_queue_submit_envelope: msgid=0, "
 			    "evpid=%016"PRIx64, evp.id);
 		ret = queue_envelope_create(&evp);
-		m_create(p_pony, IMSG_QUEUE_ENVELOPE_SUBMIT, 0, 0, -1);
-		m_add_id(p_pony, reqid);
+		m_create(p_dispatcher, IMSG_QUEUE_ENVELOPE_SUBMIT, 0, 0, -1);
+		m_add_id(p_dispatcher, reqid);
 		if (ret == 0)
-			m_add_int(p_pony, 0);
+			m_add_int(p_dispatcher, 0);
 		else {
-			m_add_int(p_pony, 1);
-			m_add_evpid(p_pony, evp.id);
+			m_add_int(p_dispatcher, 1);
+			m_add_evpid(p_dispatcher, evp.id);
 		}
-		m_close(p_pony);
+		m_close(p_dispatcher);
 		if (ret) {
 			m_create(p_scheduler,
 			    IMSG_QUEUE_ENVELOPE_SUBMIT, 0, 0, -1);
@@ -177,10 +174,10 @@ queue_imsg(struct mproc *p, struct imsg *imsg)
 		m_msg(&m, imsg);
 		m_get_id(&m, &reqid);
 		m_end(&m);
-		m_create(p_pony, IMSG_QUEUE_ENVELOPE_COMMIT, 0, 0, -1);
-		m_add_id(p_pony, reqid);
-		m_add_int(p_pony, 1);
-		m_close(p_pony);
+		m_create(p_dispatcher, IMSG_QUEUE_ENVELOPE_COMMIT, 0, 0, -1);
+		m_add_id(p_dispatcher, reqid);
+		m_add_int(p_dispatcher, 1);
+		m_close(p_dispatcher);
 		return;
 
 	case IMSG_SCHED_ENVELOPE_REMOVE:
@@ -215,7 +212,7 @@ queue_imsg(struct mproc *p, struct imsg *imsg)
 
 		bounce.type = B_FAILED;
 		envelope_set_errormsg(&evp, "Envelope expired");
-		envelope_set_esc_class(&evp, ESC_STATUS_TEMPFAIL);
+		envelope_set_esc_class(&evp, ESC_STATUS_PERMFAIL);
 		envelope_set_esc_code(&evp, ESC_DELIVERY_TIME_EXPIRED);
 		queue_bounce(&evp, &bounce);
 		queue_log(&evp, "Expire", "Envelope expired");
@@ -254,9 +251,9 @@ queue_imsg(struct mproc *p, struct imsg *imsg)
 			return;
 		}
 		evp.lasttry = time(NULL);
-		m_create(p_pony, IMSG_QUEUE_DELIVER, 0, 0, -1);
-		m_add_envelope(p_pony, &evp);
-		m_close(p_pony);
+		m_create(p_dispatcher, IMSG_QUEUE_DELIVER, 0, 0, -1);
+		m_add_envelope(p_dispatcher, &evp);
+		m_close(p_dispatcher);
 		return;
 
 	case IMSG_SCHED_ENVELOPE_INJECT:
@@ -279,9 +276,9 @@ queue_imsg(struct mproc *p, struct imsg *imsg)
 			return;
 		}
 		evp.lasttry = time(NULL);
-		m_create(p_pony, IMSG_QUEUE_TRANSFER, 0, 0, -1);
-		m_add_envelope(p_pony, &evp);
-		m_close(p_pony);
+		m_create(p_dispatcher, IMSG_QUEUE_TRANSFER, 0, 0, -1);
+		m_add_envelope(p_dispatcher, &evp);
+		m_close(p_dispatcher);
 		return;
 
 	case IMSG_CTL_LIST_ENVELOPES:
@@ -529,7 +526,7 @@ queue_imsg(struct mproc *p, struct imsg *imsg)
 		return;
 	}
 
-	errx(1, "queue_imsg: unexpected %s imsg", imsg_to_str(imsg->hdr.type));
+	fatalx("queue_imsg: unexpected %s imsg", imsg_to_str(imsg->hdr.type));
 }
 
 static void
@@ -669,7 +666,7 @@ queue(void)
 	config_peer(PROC_CONTROL);
 	config_peer(PROC_LKA);
 	config_peer(PROC_SCHEDULER);
-	config_peer(PROC_PONY);
+	config_peer(PROC_DISPATCHER);
 
 	/* setup queue loading task */
 	evtimer_set(&ev_qload, queue_timeout, &ev_qload);
@@ -679,7 +676,7 @@ queue(void)
 
 #if HAVE_PLEDGE
 	if (pledge("stdio rpath wpath cpath flock recvfd sendfd", NULL) == -1)
-		err(1, "pledge");
+		fatal("pledge");
 #endif
 
 	event_dispatch();
